// $Id: Watchdog.java,v 1.5 2003/03/14 17:13:27 lw Exp $
package team5.base.comm;

import java.util.Timer;
import java.util.TimerTask;

/**
 * A class that implements a timeout facility to interupts stalled threads.
 *
 */

public class Watchdog {
    Timer timer;
    boolean enabled;
    Thread watched;

    /**
     * The Watchdog constructor.
     * @param watched The thread that you want to watch.
     */
    public Watchdog(Thread watched) {
        this.watched = watched;
    }

    /**
     * The Watchdog constructor that also calls the enable method
     * @param watched The thread that you want to watch.
     * @param seconds The timeout value.
     */
    public Watchdog(Thread watched, long seconds) {
        enabled = true;
        this.watched = watched;
        enable(seconds);
    }

    /**
     * Set the watchdog to watch for a number of seconds.
     * @param seconds The number of seconds before the thread is interupted.
     */
    public void enable(long seconds) {
        enabled = false;
        timer = new Timer(true);
        timer.schedule(new RemindTask(), seconds * 1000);
    }

    /**
     * Set the watchdog to watch for a number of seconds repeating every cycle..
     * @param seconds The number of seconds before the thread is interupted.
     * @param cycle The number of seconds to clcle
     */
    public void enable(long seconds, long cycle) {
        enabled = false;
        timer = new Timer(true);
        timer.schedule(new RemindTask(), seconds * 1000, cycle);
    }

    /**
     * Stop the watchdog from interupting.
     */
    public void disable() throws InterruptedException {
        enabled = false;
    }

    /**
     * Cancel the timer.
     */
    public void cancel() {
        if (timer != null) {
            timer.cancel();
        }
    }

    /**
     * An inner class that interupts a thread if the "enabled" value is true
     */
    class RemindTask extends TimerTask {
        public void run() {
            if (!enabled) {
                watched.interrupt();
            }
            timer.cancel(); //Terminate the timer thread
        }
    }

    /**
     * The test method for the Watchdog class
     */
    public static void main(String args[]) {
        long sleeptime;
        if (args.length > 0) {
            sleeptime = (new Integer(args[0])).intValue();
        } else {
            sleeptime = 20;
        }
        WDTest test = new WDTest(sleeptime);
        test.start();
        System.out.println("Task scheduled.");
    }
}

/**
 * A private class used by the Watchdog test method
 */
class WDTest extends Thread {
    long sleeptime;

    WDTest(long sleeptime) {

        this.sleeptime = sleeptime;
    }

    public void run() {

        System.out.println("About to sleep for " + sleeptime + " seconds");
        Watchdog wd = new Watchdog(this, 10);
        try {
            this.sleep(sleeptime * 1000);
            wd.disable();
        } catch (java.lang.InterruptedException ie) {
            System.out.println("Thread interupted");
        }
        System.out.println("TestDone");
    }
}
