// $Id: Card.java,v 1.2 2003/02/23 21:27:41 etwilson Exp $
package team5.base.game;

/**
 * A class to encapsulate the atributes of a a poker card.
 */
public class Card {

    byte value;
    final static String[] suits = {"Clubs", "Diamonds", "Hearts", "Spades"};
    final static String[] ranks = {"Ace", "Two", "Three", "Four", "Five", "Six", "Seven",
                                   "Eight", "Nine", "Ten", "Jack", "Queen", "King"};
    public final static byte MINVAL = 0;
    public final static byte MAXVAL = 51;

    public static void main(String[] args) {

        for (byte i = 0; i < MAXVAL + 1; i++) {

            System.out.println("Card# " + i + " = " + nameOf(i));
        }
    }

    /**
    * The constructor for the Card class
    * @param value The card number from 0 to 51.
    */
    public Card(byte value) {

        this.value = value;
        if (value < MINVAL || value > MAXVAL) {
            throw new IndexOutOfBoundsException();
        }
    }
    /*
    * Returns the numerical value of the card: 0 - 51.
    * @return value The internal value of the object.
    */
    public byte getValue() {
        return (value);
    }
    /*
    * Retrives the rank of the card from Ace to King.
    * @return The string representing the rank of the card.
    */
    public String getRank() {

        return (ranks[value / 4]);
    }

    /*
    * Retrives the suit of the card from Diamond, Club,Spade or Heart.
    * @return The suit of the card.
    */
    public String getSuit() {

        return (suits[value % 4]);
    }

    /*
    * Retrives the full name of the card.
    * @return The long name of the card (ex. two of hearts).
    */
    public String getName() {

        return (rankOf(value) + " of " + suitOf(value));
    }

    /*
    * The static version of getRank.
    * @return The string representing the rank of the card.
    */
    static public String rankOf(byte value) {

        return (ranks[value / 4]);
    }

    /*
    * The static version of getSuit.
    * @return The suit of the card.
    */
    static public String suitOf(byte value) {

        return (suits[value % 4]);
    }

    /*
    * The static version of getName.
    * @return The long name of the card (ex. two of hearts).
    */
    static public String nameOf(byte value) {

        return (rankOf(value) + " of " + suitOf(value));
    }


}
