// $Id: BankImpl.java,v 1.10 2003/03/10 21:02:07 lw Exp $
package team5.casino;

import team5.base.comm.BankPOA;
import team5.base.comm.InvalidObjectID;
import team5.base.comm.InvalidQuantity;
import team5.base.comm.InvalidSequenceNumber;
import team5.casino.db.BankDB;
import team5.base.comm.Verbose;

/**
 * This class
 * TODO: write documentation
 */
public class BankImpl extends BankPOA {

    // Public methods

    /**
     * Create a new BankImpl object.
     * @param bankDB the bank database reference
     */
    public BankImpl(BankDB bankDB) {

        this.bankDB = bankDB;
    } // BankImpl

    /**
     * This method is to add the chips to the databse
     * for the specified player ID. It is assumed that the
     * bank will allow player to buy any chips.
     * @param oid the object id of the request initiator
     * @param snr the sequence number of this request
     * @param quantity  The chip quantity that player wants to buy
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     * @throws InvalidQuantity
     */
    public void buyChips(int oid, int snr, int quantity)
            throws InvalidObjectID, InvalidSequenceNumber, InvalidQuantity {

        if (quantity < 0) {
            throw new InvalidQuantity("Negative chip quantity requested",
                    quantity);
        }

        int newBalance = bankDB.buyChips(oid, snr, quantity);
        Verbose.println(
                "BI-Player " + oid + " has now " + newBalance + " chips.");
    } // buyChips

    /**
     * This method is to sell some of the chips of the specified player
     * from the bank. The result is to subtract from the chip balance
     * of the player from the bank.
     * @param oid the object id of the request initiator
     * @param snr the sequence number of this request
     * @param quantity  The chip quantity that player wants to buy
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     * @throws InvalidQuantity
     */
    public void sellChips(int oid, int snr, int quantity)
            throws InvalidObjectID, InvalidSequenceNumber, InvalidQuantity {

        int newBalance;

        if (quantity < 0) {
            throw new InvalidQuantity("Negative chip quantity requested",
                    quantity);
        }

        newBalance = bankDB.subtractChips(oid, snr, quantity);
        Verbose.println(
                "BI-Player " + oid + " has now " + newBalance + " chips.");
    } // sellChips

    /**
     * This method is to cash the chips of the specified player
     * from the bank. The result is to remove the chip balance
     * of the player from the bank.
     * @param oid the object id of the request initiator
     * @param snr the sequence number of this request
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     */
    public void sellAllChips(int oid, int snr)
            throws InvalidObjectID, InvalidSequenceNumber {

        // Get the current chip balance for the player
        int currentChips = bankDB.getChips(oid);

        // Subtract that number of chips from the DBManager
        if (bankDB.subtractChips(oid, snr, currentChips) == 0) {
            Verbose.println("BI-Player " + oid + " has cashed out.");
        } // end if

        // Remove the player from Bank database
        bankDB.removePlayer(oid);
    } // sellAllChips

    /**
     * This method is to get the current chip balance of the
     * specified player from the database.
     * @param oid the object id of the request initiator
     * @param snr the sequence number of this request
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     */
    public int getChipBalance(int oid, int snr)
            throws InvalidObjectID, InvalidSequenceNumber {

        return (bankDB.getChips(oid));
    } // getChipBalance

    /**
     * This method is to add the chips to the databse
     * for the specified player ID. It is assumed that the
     * bank will allow player to buy any chips.
     * @param oid the object id of the request initiator
     * @param snr the sequence number of this request
     * @param quantity  The chip quantity that player wants to buy
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     * @throws InvalidQuantity
     */
    public void addChips(int oid, int snr, int quantity)
            throws InvalidObjectID, InvalidSequenceNumber, InvalidQuantity {

        int newBalance = bankDB.addChips(oid, snr, quantity);
        Verbose.println(
                "BI-Player " + oid + " has now " + newBalance + " chips.");
    } // buyChips


    // Fields

    /** The vaklue of one chip. Currently, one chip is $1. */
    private static final int chipValue = 1;

    /** The bank database. */
    private BankDB bankDB;
} // Table
