// $Id: FloorImpl.java,v 1.21 2003/03/15 18:42:48 lw Exp $
package team5.casino;

import team5.base.comm.*;
import team5.base.UnhandledException;
import team5.casino.db.PlayerDB;
import team5.casino.db.FloorDB;

import java.util.Properties;

import org.omg.CORBA.ORB;
import org.omg.CORBA.UserException;
import org.omg.CORBA.SystemException;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.CosNaming.NamingContextPackage.NotFound;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.InvalidName;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

/**
 * The "floor" of the casino, containing many tables
 * <ul><li>Category: Baseline (fulfills requirement #1)</li>
 * <li>Behavior: This interface is the main entrance into the casino.</li>
 * <li>Inherits From:</li>
 * <li>Exceptions: CORBA System exceptions</li>
 * <li>Clients: Player</li></ul>
 *
 * @author Team 5
 * @version $Revision: 1.21 $
 */
public class FloorImpl extends FloorPOA {

    // Public methods

    /**
     * The constructor for the Floor class.
     * @param playerDB An instance of the Player database class.
     * @param bank The bank object used to buy and sell chips..
     * @param tables The tables object used to join a game..
     */
    public FloorImpl(NamingContextExt nsRef, FloorDB floorDB,
                     PlayerDB playerDB, Bank bank, Tables tables) {

        this.bank = bank;
        this.tables = tables;
        this.playerDB = playerDB;
        this.floorDB = floorDB;
        this.nsRef = nsRef;
    }

    /**
     * A ping class that needs to do nothing but return
     */
    public void ping() {
    }

    public void activate() {
        //Put activate stuff in here

        int[] playerIDList = floorDB.getAllPlayerIDs();
        Verbose.println("There are " + playerIDList.length +
                " players in bank DB.");
        for (int i = 0; i < playerIDList.length; i++) {
            // Buy the chips for each player
            int chips = floorDB.getChips(playerIDList[i]);

            // Get the state
            int state = floorDB.getState(playerIDList[i]);
            System.out.println("Player: " + playerIDList[i] +
                    " has state=" + state +
                    " , has chips= " + chips);

            switch (state) {
                case 2:    // Finish rejoin
                    System.out.println("In Floor: case 2 **");
                    rejoinTable(playerIDList[i], state);
                    break;
                case 3:
                    System.out.println("In Floor: case 3 ***");
                    rejoinTable(playerIDList[i], state);
                    tables.setPlayerCards(playerIDList[i]);
                    int[] gameID = playerDB.getGameID(playerIDList[i]);
                    // Because there is only one player per game, we can
                    // hard code to use the gameID[0]
                    int dealerID = 10000 + gameID[0];
                    tables.setPlayerCards(dealerID);

                    // VT: I don't know it cannot call setBetAmount here.
                    // For the dirty fix, I move to call this getBet inside
                    // the TablesImple itself after the setPlayerCards(...)
                    // Get the bet Amount
//		    int tempBet = floorDB.getBet( playerIDList[i] );
//		    tables.setBetAmount( playerIDList[i], tempBet );

                    break;

            } // end switch
        } // end for
    } // end activate

    public void shutdown() {

        Server.shutdown();
    }

//    public void getPlayerCards(int playerID) {
//        int[] cards = floorDB.getPlayerCards(playerID);
//        Table table = ((TablesImpl) tables).getCurrentTable();
//        for (int i = 0; i < cards.length; i++) {
//            table.setCard(cards[i], playerID);
//        } // end for
//
//    } // end getPlayerCards


    public boolean rejoinTable(int playerID, int state) {

        boolean rejoin_status = false;

        try {
            Integer tempInt = new Integer(playerID);
            String id = tempInt.toString();
            Player playerRef = PlayerHelper.narrow(nsRef.resolve_str(id));

            tables.joinTable(playerID, state, playerRef);
        } catch (NotFound e) {
        } catch (CannotProceed e) {
        } catch (InvalidName e) {
        } catch (InvalidObjectID e) {
        } catch (InvalidSequenceNumber e) {
        } catch (InvalidPlayer e) {
        }

        return rejoin_status;

    } // end rejoinTable


    // Floor operations

    /**
     *
     */
    public int getOID() {

        return floorDB.getNewID();
    }

    /**
     * The new or returning player registers with the system to start
     * <ul><li>Category: Baseline</li>
     * <li>Behavior: A player client uses this method to register for
     *               a session at the casino server.</li>
     * <li>Clients: Player</li></ul>
     * @param oid the object id of the request initiator
     * @param snr the sequence number of this request
     * @param playerName The client's Player interface.
     * @return the player's ID number
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     */
    public int register(int oid, int snr, String playerName)
            throws InvalidObjectID, InvalidSequenceNumber, InvalidPlayerName {

        if (playerName.length() > MAXNAMESIZE) {
            throw new InvalidPlayerName(playerName);
        }
        int playerID = playerDB.addPlayer(oid, snr, playerName);
        if (playerID == -1) {
            throw new InvalidObjectID(playerID);
        }
        Verbose.println("Floor: Added player " + playerName +
                " to database assigning player #:" + playerID);
        return playerID;
    }

    /**
     * The player requests the Bank interface.
     * <ul><li>Category: Baseline</li>
     * <li>Behavior: A player client uses this method to get the interface
     *           for banking allowing him/her to buy and sell chips.</li>
     * <li>Clients: Player</li></ul>
     * @param oid the object id of the request initiator
     * @param snr the sequence number of this request
     * @return the Banking interface
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     */
    public Bank getBank(int oid, int snr)
            throws InvalidObjectID, InvalidSequenceNumber {

        if (playerDB.getMode(oid) == -1) {
            throw new InvalidObjectID(oid);
        }
        return bank;
    }

    /**
     * The player requests the Tables interface.
     * <ul><li>Category: Baseline</li>
     * <li>Behavior: A player client uses this method to get the interface
     *           to allow him/her to join a game</li>
     * <li>Clients: Player</li></ul>
     * @param oid the object id of the request initiator
     * @param snr the sequence number of this request
     * @return the Tables interface
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     */
    public Tables getTables(int oid, int snr)
            throws InvalidObjectID, InvalidSequenceNumber {

        if (playerDB.getMode(oid) == -1) {
            throw new InvalidObjectID(oid);
        }
        return tables;
    }

    /**
     * The player asks to leave the game.
     * <ul><li>Category: Baseline</li>
     * <li>Behavior: A player client uses this method to unregister for a
     *     session from the casino server.</li>
     * <li>Clients: Player</li></ul>
     * @param oid the object id of the request initiator
     * @param snr the sequence number of this request
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     */
    public void unregister(int oid, int snr)
            throws InvalidObjectID, InvalidSequenceNumber {

        String playerName = playerDB.getLoginName(oid);
        if (playerName == "Player not in DB") {
            throw new InvalidObjectID(oid);
        }
        playerDB.removePlayer(playerName);
    }


    // Fields

    /** */
    private final int MAXNAMESIZE = 100;

    /** */
    private FloorDB floorDB = null;

    /** */
    private PlayerDB playerDB = null;

    /** */
    private Bank bank = null;

    /** */
    private Tables tables = null;

    /** */
    private NamingContextExt nsRef;
}


