// $Id: Server.java,v 1.23 2003/03/15 18:42:48 lw Exp $
package team5.casino;

import gnu.getopt.Getopt;
import org.omg.CORBA.ORB;
import org.omg.CORBA.UserException;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import team5.base.comm.*;
import team5.casino.db.BankDB;
import team5.casino.db.FloorDB;
import team5.casino.db.PlayerDB;
import team5.casino.db.TableDB;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

/**
 * This class must be run to start up the casino server.
 *
 * @author Team 5
 * @version $Revision: 1.23 $
 */
public class Server {

    // Public methods

    /**
     * Main method for Server class
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Connects to the orb,
     * creates all the servant objects.</li>
     * <li>Clients: Players</li></ul>
     * @param args The command line arguments.
     */
    public static void main(String[] args) {

        Properties props;
        namingContext = null;

        serverNum = "";
        parseArgs(args);
        props = loadProps();

        Verbose.println("\n***********************************");
        Verbose.println("* Virtual Casino Blackjack Server *");
        Verbose.println("***********************************\n");

        // Setup Shutdown thread to catch kill signals
        Runtime rt = Runtime.getRuntime();
        ShutdownThread shutdownThread = new ShutdownThread();
        rt.addShutdownHook(shutdownThread);

        // ORB initialization
        orbhost = props.getProperty("org.omg.CORBA.ORBInitialHost");
        orbport = props.getProperty("org.omg.CORBA.ORBInitialPort");
        if (orbhost == null || orbport == null) {
            System.err.println("Fatal: Orb host and port not specified");
            System.exit(-1);
        }

        ORB orb = ORB.init(args, props);
        Verbose.println("---Created ORB instance.");
        POA rootPOA = null;

        try {
            //Instantiate Servant and create reference
            rootPOA = POAHelper.narrow(
                    orb.resolve_initial_references("RootPOA"));
        } catch (org.omg.CORBA.ORBPackage.InvalidName ie) {
            System.err.println("Fatal: Unable to resolve rootPOA");
            System.exit(-1);
        }

        //Setup database information
        String user = props.getProperty("database.user");
        String password = props.getProperty("database.password");
        if (user == null || password == null) {
            System.err.println("Warning: No database user/password found using defaults");
            user = "tester";
            password = "my1dbIS?";
        }
        Verbose.println("---Using database username of " + user);
        Verbose.println("---Using database pasword of " + password);

        //Instantiate Floor Database Object.
        FloorDB fdb = new FloorDB(user, password);
        Verbose.println("---Created Floor db object.");

        //Instantiate Player Database Object.
        PlayerDB pdb = new PlayerDB(user, password);
        Verbose.println("---Created Player db object.");

        //Instantiate Bank Database Object.
        BankDB bdb = new BankDB(user, password);
        Verbose.println("---Created Bank db object.");

        //Instantiate Table Database Object.
        TableDB tdb = new TableDB(user, password);
        Verbose.println("---Created Table db object.");

        try {
            //Find NameService
            Verbose.println("---Looking for naming service on " +
                    orbhost + ":" + orbport);
            namingContext =
                    NamingContextExtHelper.narrow(
                            orb.resolve_initial_references("NameService"));
            Verbose.println("---Resolved NameService.");
        } catch (org.omg.CORBA.COMM_FAILURE ce) {
            System.err.println("Fatal: Unable to contact naming service on "
                    + orbhost + ":" + orbport);
            System.exit(-1);
        } catch (org.omg.CORBA.ORBPackage.InvalidName ce) {
            System.err.println("Fatal: Unable to resolve NameService");
            System.exit(-1);
        }

        try {

            //Instantiate Bank Object
            BankImpl bankImpl = new BankImpl(bdb);
            rootPOA.activate_object(bankImpl);
            Bank bRef = BankHelper.
                    narrow(rootPOA.servant_to_reference(bankImpl));
            Verbose.println("---Created Bank object.");

            //Instantiate Tables Object and create a holder class
            TablesImpl tablesImpl = new TablesImpl(fdb, pdb, tdb, bdb);
            rootPOA.activate_object(tablesImpl);
            Tables tRef = TablesHelper.narrow(
                    rootPOA.servant_to_reference(tablesImpl));
            Verbose.println("---Created Tables object.");

            //Instantiate Floor Object and insert it into naming service
            FloorImpl floorImpl =
                    new FloorImpl(namingContext, fdb, pdb, bRef, tRef);
            rootPOA.activate_object(floorImpl);
            Floor fRef = FloorHelper.narrow(
                    rootPOA.servant_to_reference(floorImpl));

            nc = namingContext.to_name("Floor" + serverNum);
            if (force == false) {
                try {
                    namingContext.resolve(nc);
                    System.err.println(
                            "Fatal: Floor interface is already in the name service, " +
                            "\n       another Blackjack server must be running.");
                    System.exit(1);
                } catch (UserException e) {
                }
            }
            namingContext.rebind(nc, fRef);
            shutdownThread.addName(namingContext, nc);
            Verbose.println("---Added name Floor" + serverNum + ".");

            //--------------------------------------------------------
            // Activate rootpoa
            //--------------------------------------------------------
            rootPOA.the_POAManager().activate();

            // VT: For testing. It should be called by replication manager
            //            System.out.println("Calling Activate ");
            //            floorImpl.activate();

            //--------------------------------------------------------
            // Sit and wait for connections
            //--------------------------------------------------------
            System.out.println("Info: Server ready and running .....");
            orb.run();


        } catch (UserException e) {
            System.err.println("Fatal: CORBA exception");
            //e.printStackTrace();
            System.exit(1);
        }
    }

    public static void shutdown() {

        Verbose.println("---Replication Manager requested shutdown....");
        if (nc != null && namingContext != null) {
            Verbose.print("---Attempting to remove reference from "
                    + "naming server....");
            try {
                namingContext.unbind(nc);
            } catch (Exception e) {
                Verbose.println("---Couldn't remove name from service.");
            }
        }
        Verbose.println("done.");
    }


    // Private methods

    private static void parseArgs(String[] argv) {

        Verbose.verbose = false;
        int c;
        Getopt g = new Getopt("Server", argv, "fhn:p:v");

        while ((c = g.getopt()) != -1) {
            switch (c) {
                case 'f':
                    force = true;
                    break;
                case 'h':
                    usage();
                    break;
                case 'n':
                    serverNum = g.getOptarg();
                    break;
                case 'p':
                    propfileName = g.getOptarg();
                    break;
                case 'v':
                    System.out.println("Info: Setting Verbose Mode");
                    Verbose.verbose = true;
                    break;
                default:
                    System.err.println("Error: Unknown Option " + g.getOptarg());
                    usage();
            }
        }
    }

    /**
     * Load the properties file from disk.
     * @return a properties object.
     */
    private static Properties loadProps() {

        Properties props = null;

        Verbose.println("---Loading properties file: " + propfileName);
        try {
            props = new Properties(System.getProperties());
            props.load(new BufferedInputStream(new FileInputStream(propfileName)));
        } catch (FileNotFoundException e) {
            System.err.println("Fatal: Properties file: " + propfileName + " not found");
            System.exit(-1);
        } catch (IOException e) {
            System.err.println("Fatal: Could not read properties file: " + propfileName);
            System.exit(-1);
        }
        return props;
    }

    /**
     * Printout the commandline options.
     */
    private static void usage() {

        System.out.println("Usage: java -classpath $CLASSPATH team5.casino.Server");
        System.out.println("-f Force the registration into the nameserver.");
        System.out.println("-h Print this message.");
        System.out.println("-n <ServerNum> Set the server instance number.");
        System.out.println("-p <prop file name> Set the properties file");
        System.out.println("-v Set the output to verbose");
        System.exit(0);
    }

    private static boolean force;
    private static String serverNum;
    private static String propfileName;
    private static String orbhost;
    private static String orbport;
    private static NamingContextExt namingContext;
    private static NameComponent[] nc;
}
