// $Id: TablesImpl.java,v 1.17 2003/03/15 19:21:18 lw Exp $
// TablesImpl.java  - The implementation of the Tables interface for the
//              Virtual Casino Client/Server Application.
// Authors    - Team5
// Date       - Jan 31, 2003
//
package team5.casino;

import java.util.*;

import team5.casino.db.PlayerDB;
import team5.casino.db.TableDB;
import team5.casino.db.BankDB;
import team5.casino.db.FloorDB;
import team5.casino.comm.PlayerProxy;
import team5.base.comm.*;

/**
 * The set of gaming tables
 * <ul><li>Category:  Baseline (fulfills baseline requirment #4)</li>
 * <li>Behavior:  This interface supports methods to allow a player
 *           to join a game at a virtual table.</li>
 * <li>Clients: Player</li></ul>
 * @author Team 5
 * @version $Revision: 1.17 $
 */
public class TablesImpl extends TablesPOA {

    // Public methods

    /**
     * Constructor for TablesImpl class
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Contructs a new Tables object..</li>
     * <li>Clients: Players</li></ul>
     * @param playerDB The player database.
     * @param tableDB The tables database.
     * @param bankDB The bank database.
     */
    public TablesImpl(FloorDB floorDB, PlayerDB playerDB,
                      TableDB tableDB, BankDB bankDB) {
        this.floorDB = floorDB;
        this.playerDB = playerDB;
        this.tableDB = tableDB;
        this.bankDB = bankDB;
        players = new Players();
    }

    /**
     * Player asks to join any table.
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Enables a player to join a table.</li>
     * <li>Clients: Players</li></ul>
     * @param playerID the player ID
     * @param snr
     * @param player
     * @throws InvalidObjectID The player ID is invalid.
     * @throws InvalidSequenceNumber
     * @throws InvalidPlayer
     */
    public void joinTable(int playerID, int snr, Player player)
            throws InvalidObjectID, InvalidSequenceNumber, InvalidPlayer {

        int mode = playerDB.getMode(playerID);

        if (mode == -1) {
            throw new InvalidObjectID(playerID);
        }
        if (mode == playerDB.PLAYING) {
            Verbose.println("TI-joinTable: This playerID is in the table. Rejoin.");
            int[] gameIDList = playerDB.getGameID(playerID);
            Verbose.println("TI-PlayerID#" + playerID +
                    " rejoins to tableID#" + gameIDList[0]);

            players.put(playerID, new PlayerProxy(player));
            tbl = new Table(tableDB, bankDB, gameIDList[0], players);
            new Thread(tbl).start();
        } else {
            playerDB.setMode(playerID, playerDB.PLAYING);
            // Find a table for the player if none is available
            // create a new one.
            // NOTE: getOpenGame really returns a game that may not be open
            gameID = tableDB.getOpenGame(playerID, snr);
            tableDB.setStatus(gameID, tableDB.RUNNING);

            Verbose.println("TI-Player #" + playerID + " joined game #" + gameID);

            players.put(playerID, new PlayerProxy(player));
            Verbose.println("TI-Create the tableID#" + gameID);
            tbl = new Table(tableDB, bankDB, gameID, players);
            new Thread(tbl).start();

            // Save the state to bank DB
            bankDB.setState(playerID, 2);
            System.out.println("The state is set to be " + bankDB.getState(playerID));
        }
    }

    /**
     * Player elects to leave a game.
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Enables a player to leave a table.</li>
     * <li>Clients: Players</li></ul>
     * @param playerID The players ID.
     * @param snr
     * @throws InvalidObjectID
     * @throws InvalidSequenceNumber
     */
    public void leaveTable(int playerID, int snr)
            throws InvalidObjectID, InvalidSequenceNumber {

        if (playerDB.getMode(playerID) == -1) {
            throw new InvalidObjectID(playerID);
        }
        tableDB.removePlayer(playerID);
    }

    public void setPlayerCards(int playerID) {

        int[] cards = floorDB.getPlayerCards(playerID);

        System.out.println("TablesImpl:setPlayerCards is called. Player#" + playerID + " has " + cards.length + " cards.");

        for (int i = 0; i < cards.length; i++) {
            tbl.setCard(cards[i], playerID);
            System.out.println("Player#" + playerID + "has card#" + cards[i]);
        }

        // VT:
        // For dirty fix, I call get bet here
        int tempBet = floorDB.getBet(playerID);
        setBetAmount(playerID, tempBet);
    }

    /**
     */
    public void setBetAmount(int playerID, int bet) {

        tbl.putBetAmount(playerID, bet);
    } // end setBetAmount

    /**
     This method is to get the table reference.
     */
    public Table getCurrentTable() {

        return tbl;
    } // end getCurrentTable


    // Fields

    /** The floor database. */
    private FloorDB floorDB;

    /** The player database.*/
    private PlayerDB playerDB;

    /** The table database. */
    private TableDB tableDB;

    /** The bank database. */
    private BankDB bankDB;

    /** The ID of the game at this table. */
    private int gameID;

    /** The internal table holding playerIDs and player proxies. */
    private Players players;

    /** The place to hold the table object */
    private Table tbl;
}



