// $Id: PlayerApp.java,v 1.14 2003/03/14 06:23:19 jerobert Exp $
package team5.player;

import java.io.*;
import java.util.*;
import gnu.getopt.Getopt;

import team5.base.comm.*;

/**
 * This class must be started to run the player client application.
 *
 * @author Team 5
 * @version $Revision: 1.14 $
 */
public class PlayerApp {

    // Public methods

    public static void main(String args[]) {

        parseArgs(args);
        boolean exitFlag = false;
        boolean quitFlag = false;
        int menuChoice = 0;
        String tempString = "";
        Integer myInteger = new Integer(0);
        Properties props = null;
        PlayerServerThread playerServerThread = null;
        PlayerSignals playerSignals = new PlayerSignals();
        Player playerRef = null;
        PlayerClient playerClient = null;
        BufferedReader MyReader = new BufferedReader(new InputStreamReader(System.in));

        System.out.println("Starting the Player Application...");

        try
        {
            // Set properties
            props = loadProps(propfileName);

            // Start the player client
            playerClient = new PlayerClient(args, props, playerSignals);

            // Create the player server thread
            playerServerThread = new PlayerServerThread(args, props, playerSignals);
            playerRef = playerServerThread.getPlayerRef();

            // Register the player
            System.out.println("...Registering the Player with the Floor");
            if (playerClient.registerPlayer(playerRef) == true) {
                // There was an error
                System.err.println("ERROR:Could not register with the Floor");
                System.exit(1);
            }

            // Buy chips
            System.out.println("...Buy chips from the Bank");
            // Buy chips from the Bank
            if (playerClient.buyChips() == true) {
                // There was an error
                System.err.println("ERROR:Could not buy chips");
            }
            playerServerThread.setChips(playerClient.GetNumberOfChips());

        } catch (Exception e) {
            System.err.println("EXCEPTION-PlayerApp startup: " + e);
            System.exit(1);
        }

        // Main menu loop
        while (quitFlag != true) {
            try {
                System.out.println("Player Menu");
                System.out.println(" 0 - Quit the game");
                System.out.println(" 1 - Buy chips from the Bank");
                System.out.println(" 2 - Join a Table and begin a game");
                System.out.println(" Enter a number to select a menu option: ");
                tempString = MyReader.readLine();
                myInteger = Integer.valueOf(tempString);

                switch (myInteger.intValue()) {
                    case 0:
                        System.out.println("User Command: Quit the Game");
                        quitFlag = true;
                        break;
                    case 1:
                        System.out.println("User Command: Buy Chips");
                        // Buy chips from the Bank
                        if (playerClient.buyChips() == true) {
                            // There was an error
                            System.err.println("ERROR:Could not buy chips");
                        }
                        playerServerThread.setChips(playerClient.GetNumberOfChips());
                        break;
                    case 2:
                        System.out.println("User Command: Join a Table");
                        // Join a Table
                        playerServerThread.start();
                        if (playerClient.joinTable(playerRef) == true) {
                            // There was an error
                            System.err.println("ERROR:Could not join a Table");
                        }
                        System.out.println("...Player joined a Table");
                        // Yield to Server Thread
                        System.out.println("...Waiting for dealer to start game");
                        Thread t = Thread.currentThread();
                        exitFlag = false; // reset the flag
                        while (exitFlag == false) {
                            t.yield();

                            // Read signal from PlayerServer
                            //  Exit signal = 1000
                            if (playerSignals.getSignal() == 1000) {
                                //Received exit signal
                                exitFlag = true;
                            }
                        }
                        // Leave a Table
                        System.out.println("...Player leaving a Table");
                        if (playerClient.leaveTable() == true) {
                            // There was an error
                            System.err.println("ERROR:Could not leave a Table");
                        }
                        playerServerThread.StopService();
                        break;
                    default:
                        System.out.println("Not a valid Player command. ");
                        break;
                }
            } catch (NumberFormatException NFerrer) {
                System.err.println("EXCEPTION:Invalid number");
            } catch (Exception e) {
                System.err.println("EXCEPTION-PlayerApp menu: " + e);
            }
        }

        // Sell chips to the Bank
        System.out.println("...Player will sell all chips to the Bank");
        if (playerClient.sellChips() == true) {
            // There was an error
            System.err.println("Player could not sell chips to the Bank");
        }

        // Unregister from Floor
        System.out.println("...Player will unregister from the Floor");
        if (playerClient.unregisterPlayer() == true) {
            // There was an error
            System.err.println("Player could not unregister from the Floor");
        }
        System.out.println("SUCCESS ->Player shutdown is complete");
    }

    //private methods

    private static void parseArgs(String[] argv) {

        Verbose.verbose = false;
        int c;
        Getopt g = new Getopt("Server", argv, "hp:v");

        while ((c = g.getopt()) != -1) {
            switch (c) {
                case 'h':
                    usage();
                    break;
                case 'p':
                    propfileName = g.getOptarg();
                    break;
                case 'v':
                    System.out.println("---Setting Verbose Mode");
                    Verbose.verbose = true;
                    break;
                default:
                    System.out.println("Unknown Option " + g.getOptarg());
                    usage();
            }
        }
    }

    
    /**
     * Load the properties file from disk.
     * @return a properties object.
     */
    private static Properties loadProps(String propfileName) {
        Properties props = null;
        Verbose.println("---Loading properties file: " + propfileName);
        try {
            props = new Properties(System.getProperties());
            props.load(new BufferedInputStream(new FileInputStream(propfileName)));
        } catch (Exception e) {
            System.out.println("Properties file: " + propfileName + " not found");
            System.exit(-1);
        }
        return props;
    }

    /**
     * Printout the commandline options.
     */
    private static void usage() {

        System.out.println("Usage: java -classpath $CLASSPATH team5.casino.PlayerApp");
        System.out.println("-h Print this message.");
        System.out.println("-p <prop file name> Set the properties file");
        System.out.println("-v Set the output to verbose");
        System.exit(0);
    }
    
    //private variables
    private static String propfileName;


}
