// $Id: PlayerClient.java,v 1.13 2003/03/14 06:23:19 jerobert Exp $
package team5.player;

import java.io.*;
import java.util.*;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import team5.player.comm.*;
import team5.base.comm.*;

/**
 * TODO: Documentation
 * @author Team 5
 * @version $Revision: 1.13 $
 */
public class PlayerClient {

    // This is configuration information specific to the client
    private String myArgs[];
    private Properties myProps;
    private PlayerSignals myPlayerSignals = null;

    // This is system state information detected by the client
    private NamingContextExt nsRef = null;
    private IFloorProxy floorRef = null;
    private IBankProxy bankRef = null;
    private ITablesProxy tablesRef = null;

    // This is system state information set by the client
    private String playerName = "";

    // This is client state information set by the Floor
    private int playerID = 0;
    private int numberOfChips = 0;

    // This is client state information set by the Table/Dealer


    /**
     * This constructor method is to create a player.
     * @param args The command-line argument to create the ORB
     */
    public PlayerClient(String[] args, Properties props, PlayerSignals playerSignals) {
        try {
            System.out.println("\n");
            System.out.println("***************************************");
            System.out.println("** Virtual Casino Player Application **");
            System.out.println("***************************************");
            System.out.println(" ---Starting the Player Application...");

            this.myArgs = args;
            this.myProps = props;
            this.myPlayerSignals = playerSignals;

            // Start the player server
            //System.out.println("...Starting the Player Server...");
            //if (startPlayerServer() == true)
            //{
            //    System.err.println("Player server could not be started...Exiting");
            //}

        } catch (Exception e) {
            System.err.println("EXCEPTION: PlayerClient constructor : " + e);
            e.printStackTrace(System.out);
        }
    }

    /**
     * This method is to register the player with the floor.
     * @return True if the player cannot register. False, otherwise.
     * The PlayerName length is checked (1 - 100)
     */
    public boolean registerPlayer(Player playerRef) {
        String tempString = "";
        IntHolder playerIDHolder = new IntHolder(0);

        try {
            // Get the Player name
            BufferedReader MyReader =
                    new BufferedReader(new InputStreamReader(System.in));
            boolean valid;
            do {
                try {
                    System.out.println(" Q--Please enter your Player name: ");
                    tempString = MyReader.readLine();

                    // Verify PlayerName length is > 1 and < 100
                    if ((tempString.length() < 1) || (tempString.length() > 100)) {
                        System.out.println(" E--The player name length is invalid");
                        valid = false;
                    } else {
                        valid = true;
                    }
                } catch (IOException IOerror) {
                    System.err.println("IOEXCEPTION-registerPlayer: Get Player Name read exception");
                    valid = false;
                }
            } while (valid == false);
            playerName = tempString;

            System.out.println(" ---Contacting the Casino Floor...");
            Verbose.println("In PlayerClient: myProps = " + myProps);
            // Create and initialize the ORB
            ORB orb = ORB.init(myArgs, myProps);
            Verbose.println("In registerPlayer:106");

            // Find the naming servie 
            try {
                org.omg.CORBA.Object objRef =
                        orb.resolve_initial_references("NameService");
                nsRef = NamingContextExtHelper.narrow(objRef);
                System.out.println(" ---Found the naming service");
                //System.out.println(" ---Found the naming service at "+nsRef);
            } catch (org.omg.CORBA.COMM_FAILURE e) {
                System.err.println("EXCEPTION-registerPlayer: Can't contact naming service");
                return true;  // error
            }

            // Lookup the Floor 
            //floorRef = FloorProxy.getInstance();
            Verbose.println("In registerPlayer:122");
            floorRef = FTFloorProxy.getInstance(myProps);
            System.out.println(" ---Found the Floor interface");
            this.playerID = ObjectID.getOID();

            // Invoke the Floor server 
            System.out.println(" -->Registering the Player with the Floor...");
            floorRef.register(playerName);
            System.out.println(" ---Player is registered with the Floor"
                    + " with playerID = " + playerID);

            // Put the player in the naming service
            tempString = String.valueOf(this.playerID);
            NameComponent[] nc = nsRef.to_name(tempString);
            nsRef.rebind(nc, playerRef);
            System.out.println(" -->Registered the Player with the Naming Service as: " + tempString);

            // Lookup the Bank 
            //bankRef = BankProxy.getInstance();
            bankRef = FTBankProxy.getInstance(myProps);
            System.out.println(" ---Found the Bank interface");

            // Lookup the Table 
            //tablesRef = TablesProxy.getInstance();
            tablesRef = FTTablesProxy.getInstance(myProps);
            System.out.println(" ---Found the Tables interface");

            return false;  // no error

        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("CORBA_EXCEPTION-registerPlayer: " + e);
            System.err.println("-Minor code = " + e.minor);
            System.err.println("-Comp Status = " + e.completed);
            System.out.println("-Player is not registered!");
            return true;  // error
        } catch (Exception e) {
            System.err.println("EXCEPTION-registerPlayer: " + e);
            e.printStackTrace(System.err);
            System.out.println("-Player is not registered!");
            return true;  // error
        }
    }

    /**
     * This method is to unregister the player from the floor. It
     * is called when the player wants to leave the casino.
     * @return True if player fails to unregister. False, otherwise.
     */
    public boolean unregisterPlayer() {
        String tempString = "";
        try {
            // Remove the player from the naming service
            tempString = String.valueOf(this.playerID);
            NameComponent[] nc = nsRef.to_name(tempString);
            nsRef.unbind(nc);
            System.out.println(" -->Removed the Player with the Naming Service as: " + tempString);

        } catch (Exception e) {
            System.err.println("EXCEPTION-unregisterPlayer: " + e);
            System.out.println("-Player is still registered in the naming service");
        }

        try {
            // Check floorRef
            if (floorRef == null) {
                System.err.println("unregisterPlayer ERROR: floorRef is null");
                return true;  // error
            }

            floorRef.unregister();

            System.out.println(" ---Player unregistered succesfully!");
            return false;  // no error

        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("CORBA_EXCEPTION-unregisterPlayer: " + e);
            System.err.println("-Minor code = " + e.minor);
            System.err.println("-Comp Status = " + e.completed);
            System.out.println("-Player failed to unregister!");
            return true;  // error
        } catch (Exception e) {
            System.err.println("EXCEPTION-unregisterPlayer: " + e);
            e.printStackTrace(System.err);
            System.out.println("-Player failed to unregister!");
            return true;  // error
        }
    }

    /**
     * This method is to allow player to buy the chips from the bank.
     * @return True if the player fails to buy the chips. False, otherwise.
     * The number of chips to buy is checked (10 - 50000)
     */
    public boolean buyChips() {
        Integer quantity;
        try {
            // Check bankRef
            if (bankRef == null) {
                System.err.println("ERROR-buyChips: bankRef is null");
                return true;  // error
            }

            // Ask the player how many chips should be bought
            boolean valid;
            do {
                System.out.println(" Q--How many chips do you want to buy? ");
                quantity = GetInteger();

                // Verify quantity is >= 10 and <= 50000
                if ((quantity.intValue() < 10) || (quantity.intValue() > 50000)) {
                    System.out.println(" E--The quantity of chips is invalid");
                    valid = false;
                } else {
                    valid = true;
                }
            } while (valid == false);

            // Invoke the Bank server and get the chips
            System.out.println(" -->Asking the Bank to buy " + quantity + " chips...");
            bankRef.buyChips(quantity.intValue());
            numberOfChips = quantity.intValue();

            return false;  // no error

        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("CORBA_EXCEPTION-buyChips: " + e);
            System.err.println("-Minor code = " + e.minor);
            System.err.println("-Comp Status = " + e.completed);
            System.out.println("-Player did not buy chips!");
            return true;  // error
        } catch (Exception e) {
            System.err.println("EXCEPTION-buyChips: " + e);
            e.printStackTrace(System.err);
            System.out.println("-Player did not buy chips!");
            return true;  // error
        }
    }

    // -------------------------------------------------------------------
    /**
     * This method is to allow a player to sell the chips to the bank.
     * @return True if the player fails to sell the chips. False, otherwise.
     */
    public boolean sellChips() {
        try {
            // Check bankRef
            if (bankRef == null) {
                System.err.println("ERROR-sellChips: bankRef is null");
                return true;  // error
            }

            // Invoke the Bank server and sell the chips
            System.out.println(" ---Asking the Bank to sell all chips ...");
            bankRef.sellAllChips();

            return false;  // no error

        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("CORBA_EXCEPTION-sellChips: " + e);
            System.err.println("-Minor code = " + e.minor);
            System.err.println("-Comp Status = " + e.completed);
            System.out.println("-Player did not sell chips!");
            return true;  // error
        } catch (Exception e) {
            System.err.println("EXCEPTION-sellChips: " + e);
            e.printStackTrace(System.err);
            System.err.println("-Player did not sell chips!");
            return true;  // error
        }
    }

    /**
     * This method is to allow player to join the table.
     * Upon successfully joining the table, the table will
     * be assigned to the player.
     * @param playerRef
     * @return True if player fails to join the table. False, otherwise.
     */
    public boolean joinTable(Player playerRef) {
        try {
            // Check tablesRef
            if (tablesRef == null) {
                System.err.println("ERROR-joinTable: tablesRef is null");
                return true;  // error
            }

            // Invoke the Table server and join a table
            System.out.println(" ---Asking the Table server to join a table.");

            // Instatiate Servant and create reference
            //ORB orb = ORB.init(myArgs, myProps);
            //POA rootPOA = POAHelper.narrow (orb.resolve_initial_references("RootPOA"));
            // Register player reference
            System.out.println(" ---Registering the Player server reference with ID=" + playerID);
            tablesRef.joinTable(playerRef);
            System.out.println(" ---Player joined a table!");

            // Activate the ORB service
            //System.out.println("---------------------- Activating the Player Service");
            //rootPOA.the_POAManager().activate();
            //orb.run();

            return false;  // no error

        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("CORBA_EXCEPTION-joinTable: " + e);
            System.err.println("-Minor code = " + e.minor);
            System.err.println("-Comp Status = " + e.completed);
            System.out.println("-Player did not join a table!");
            return true;  // error
        } catch (Exception e) {
            System.err.println("EXCEPTION-joinTable: " + e);
            e.printStackTrace(System.err);
            System.out.println("-Player did not join a table!");
            return true;  // error
        }
    }

    /**
     * This method is called when the player wants to leave the table.
     * @return True if the player fails to leave the table. False, otherwise.
     */
    public boolean leaveTable() {
        try {
            // Check tablesRef
            if (tablesRef == null) {
                System.err.println("ERROR-leaveTable: tablesRef is null");
                return true;  // error
            }

            // Invoke the Table server and leave a table
            System.out.println(" ---Asking the Table server to leave a table");
            tablesRef.leaveTable();

            return false;  // no error

        } catch (org.omg.CORBA.SystemException e) {
            System.err.println("CORBA_EXCEPTION-leaveTable: " + e);
            System.err.println("-Minor code = " + e.minor);
            System.err.println("-Comp Status = " + e.completed);
            System.out.println("-Player did not leave a table!");
            return true;  // error
        } catch (Exception e) {
            System.err.println("EXCEPTION-leaveTable: " + e);
            e.printStackTrace(System.err);
            System.out.println("-Player did not leave a table!");
            return true;  // error
        }
    }

    /**
     * This method is to get the integer string to the integer value.
     * @return The integer value
     */
    private Integer GetInteger() {
        BufferedReader MyReader = new BufferedReader(new InputStreamReader(System.in));
        String tempString = "";
        Integer myInteger = new Integer(0);
        boolean valid = true;

        do {
            try {
                tempString = MyReader.readLine();
            } catch (IOException IOerror) {
                //System.out.println("GetInteger: Read error");
                valid = false;
            }

            try {
                myInteger = Integer.valueOf(tempString);
                valid = true;
            } catch (NumberFormatException NFerror) {
                System.out.println("Please input a number:");
//                System.out.println("GetInteger: Number format error");
                valid = false;
            }
        } while (valid == false);

        return myInteger;
    }

    /**
     * This method returnsa the number of purchased chipcs .
     * @return The integer value of chips purchased
     */
    public int GetNumberOfChips() {
        return numberOfChips;
    }
}
