// $Id: PlayerServer.java,v 1.14 2003/03/14 06:23:19 jerobert Exp $
// PlayerServer.java - This is the server portion of the Player application
//               for the Virtual Casino.
// Authors     - Team5
// Date        - Feb 4, 2003
//

package team5.player;

import java.io.*;
import java.util.*;

import team5.base.comm.*;
import team5.base.game.Card;

class PlayerServer extends PlayerPOA {

    // This is player state information set by the DEALER
    private boolean tableActive = false;
    private PlayerSignals myPlayerSignals = null;

    // This is player state information set by the DEALER
    private int myNumberOfPlayers = 0;
    private int myBalance = 0;
    private Vector myHand = new Vector();
    private Vector dealerHand = new Vector();

    public PlayerServer(PlayerSignals playerSignals) {

        super();
        this.myPlayerSignals = playerSignals;
    }

    /**
     * This method is called when the player places the bet.
     * @return
     * The amount is checked (10 - 25000)
     * The amount is checked against the current balance
     */
    public synchronized int placeBet(int oid, int snr) throws InvalidSequenceNumber {

//        if (snr < lastSNR) {
//            throw new InvalidSequenceNumber(snr);
//        }
        // ignore duplicate message
        if (snr == lastSNR) {
            return lastBet;
        }
        lastOID = oid;
        lastSNR = snr;

        System.out.println("->The DEALER says, 'Place your bet!'");
        Integer tmpInteger = new Integer(0);

        try {
            boolean valid;
            do {
                System.out.println(" Q-How many chips do you want to bet (0 to QUIT)? ");
                tmpInteger = GetInteger();

                // Verify amount is >= 10 and <= 25000
                if (tmpInteger.intValue() == 0) {
                    System.out.println(" -Placed 0 bet to QUIT");
                    valid = true;
                } else if ((tmpInteger.intValue() < 10) || (tmpInteger.intValue() > 25000)) {
                    System.out.println(" -Error: This is an invalid bet amount (between 10 and 25000)");
                    valid = false;
                } else if (tmpInteger.intValue() > myBalance) {
                    System.out.println(" -Error: You don't have that many chips");
                    valid = false;
                } else {
                    valid = true;
                }
            } while (valid == false);

        } catch (Exception e) {
            System.err.println(" -ERROR-placeBet: " + e);
            e.printStackTrace(System.err);
            System.out.println(" -The Player did not place a bet for this game");
        }
        System.out.println(" -You bet " + tmpInteger.intValue() + " chips for this game");
        lastBet = tmpInteger.intValue();
        return lastBet;
    }

    /**
     * This method is called when the player receives the card from
     * the dealer.
     * @param cardID The card ID that player receives
     */
    public synchronized void acceptCard(int oid, int snr, byte cardID)
            throws InvalidSequenceNumber {

//        if (snr < lastSNR) {
//            throw new InvalidSequenceNumber(snr);
//        }
        // ignore duplicate message
        if (snr == lastSNR) {
            return;
        }
        lastOID = oid;
        lastSNR = snr;

        try {
            Card tmpCard = new Card(cardID);

            System.out.println("->The DEALER has given you: " + tmpCard.getName());
            myHand.addElement(tmpCard);

            DisplayCards();

        } catch (IndexOutOfBoundsException eb) {
            System.err.println("EXCEPTION-acceptCard: " + eb);
            System.err.println("-the cardID was invalid");
            System.out.println("The Player did not accept a card");
        } catch (Exception e) {
            System.err.println("EXCEPTION-acceptCard: " + e);
            e.printStackTrace(System.err);
            System.out.println("The Player did not accept a card");
        }
    }

    /**
     * This method is to accept the other players' card to display.
     * @param playerID The other player ID that the card will be displayed.
     * @param cardID The card ID of the specified player
     * The dealerID is >= 10000
     */
    public synchronized void acceptOthersCard(int oid, int snr, int playerID, byte cardID)
            throws InvalidSequenceNumber {

//        if (snr < lastSNR) {
//            throw new InvalidSequenceNumber(snr);
//        }
        // ignore duplicate message
        if (snr == lastSNR) {
            return;
        }
        lastOID = oid;
        lastSNR = snr;

        try {
            Card tmpCard = new Card(cardID);

            // Check for invalid playerID
            if (playerID < 0 ) {
                System.err.println("-the playerID was invalid");
                return;
            }

            // Display card for playerID
            if (playerID >= 10000) {
                // This is the DEALER's card
                System.out.println("->The DEALER has recieved the following card: " + tmpCard.getName());
                dealerHand.addElement(tmpCard);

            } else {
                // This is another player's card
                System.out.println("->The DEALER has given player " + playerID + " the following card: " + tmpCard.getName());
                // NOTE:  This version does not store the hands of other players
            }

            DisplayCards();
        } catch (IndexOutOfBoundsException eb) {
            System.err.println("EXCEPTION-acceptOthersCard: " + eb);
            System.err.println("-the cardID was invalid");
            System.out.println("The Player did not accept another player's card");
        } catch (Exception e) {
            System.err.println("ERROR-acceptOthersCard: " + e);
            e.printStackTrace(System.err);
            System.out.println("The Player did not accept another player's card");
        }
    }

    /**
     * The method is to start the game with the given number of players.
     * @param numberOfPlayers The number of players
     */
    public synchronized void startGame(int oid, int snr, byte numberOfPlayers)
            throws InvalidSequenceNumber {

//        if (snr < lastSNR) {
//            throw new InvalidSequenceNumber(snr);
//        }
        // ignore duplicate message
        if (snr == lastSNR) {
            return;
        }
        lastOID = oid;
        lastSNR = snr;


        try {
            if ((numberOfPlayers<1) || (numberOfPlayers>5)){
                System.err.println("ERROR-startGame: the numberOfPlayers in invalid");
                System.out.println("The Player did not start a game");
                return;
            }
            System.out.println("\n--------------------------");
            System.out.println("->The DEALER is starting a game with " + numberOfPlayers + " players...");

            // Set local variables
            myNumberOfPlayers = numberOfPlayers;
            myHand.clear();
            dealerHand.clear();
        } catch (Exception e) {
            System.err.println("EXCEPTION-startGame: " + e);
            e.printStackTrace(System.err);
            System.out.println("The Player did not start a game");
        }

    }

    /**
     * This method is called when the current game finishes.
     * @param won True if the player wins. False, otherwise.
     * @param newBalance The new chip balance of the player
     */
    public synchronized void gameOver(int oid, int snr, boolean won, int newBalance)
            throws InvalidSequenceNumber {

//        if (snr < lastSNR) {
//            throw new InvalidSequenceNumber(snr);
//        }
        // ignore duplicate message
        if (snr == lastSNR) {
            return;
        }
        lastOID = oid;
        lastSNR = snr;

        try {
            // Test newBalance
            // NOTE: Does not test for artificially high balance amounts
            if (newBalance < 0) {
                System.err.println("ERROR-gameOver: the newBalance in invalid");
                System.out.println("The Player did not end a game");
                return;
            }

            System.out.println("->The DEALER is ending the game ...");
            if (won == true) {
                System.out.println("  You won the game and now have " + newBalance + " chips!\n");
            } else {
                System.out.println("  Sorry, you lost the game and now have " + newBalance + " chips!\n");
            }
            myBalance = newBalance;

        } catch (Exception e) {
            System.err.println("EXCEPTION-gameOver: " + e);
            e.printStackTrace(System.err);
            System.out.println("The Player did not end a game");
        }

    }

    // -------------------------------------------------------------------
    /**
     * This method is to display the player's option to hit or stay.
     * @return void
     */
    public synchronized int playerOption(int oid, int snr) throws InvalidSequenceNumber {

        String hitstay = null;
        int tempInt = 1; //Stay is the default

//        if (snr < lastSNR) {
//            throw new InvalidSequenceNumber(snr);
//        }
        // ignore duplicate message
        if (snr == lastSNR) {
            return lastOption;
        }
        lastOID = oid;
        lastSNR = snr;

        System.out.println("->The DEALER is asking, 'Hit or Stay ?' :");

        DisplayCards();

        BufferedReader MyReader =
                new BufferedReader(new InputStreamReader(System.in));

        try {
            System.out.println(" Q-Please enter your response <H>it or <S>tay ? :");
            hitstay = MyReader.readLine();

            if (hitstay.charAt(0) == 'h' || hitstay.charAt(0) == 'H') {
                tempInt = 0; // Hit = 0
            }

        } catch (IOException io) {
            System.err.println("EXCEPTION-playerOption: get choice option");
            System.out.println("EXCEPTION-playerOption: Assuming Stay due to error in response");
            tempInt = 1;
        }
        lastOption = tempInt;
        return lastOption;
    }

    /**
     * This method is called when the player is inactive for a
     * long period of time and dealer decides to remove this player
     * from the game.
     */
    public synchronized void kickOut(int oid, int snr) throws InvalidSequenceNumber {

//        if (snr < lastSNR) {
//            throw new InvalidSequenceNumber(snr);
//        }
        // ignore duplicate message
        if (snr == lastSNR) {
            return;
        }
        lastOID = oid;
        lastSNR = snr;

        System.out.println("->The DEALER has kicked you out of the game\n");

        try {
            myHand.clear();
            dealerHand.clear();

            myPlayerSignals.putSignal(1000);  // Send Exit signal
            tableActive = false;
        } catch (Exception e) {
            System.err.println("EXCEPTION-kickOut: " + e);
            e.printStackTrace(System.err);
            System.out.println("The Player did not kickout of a game");
        }
    }

    /**
     * This method is to get the tableActivity
     * @param activity
     */
    public synchronized void setTableActivity(boolean activity) {

        tableActive = activity;
    }

    /**
     * This method is to get the tableActivity
     * @return tableActivity
     */
    public synchronized boolean getTableActivity() {

        return tableActive;
    }

    /**
     * This method is to set the chips balance
     * @param numberOfChips number of chips
     */
    public synchronized void setChips(int numberOfChips) {

        myBalance = numberOfChips;
        Verbose.println("Balance set to " + myBalance);
    }

    /**
     * This method is to get the integer value entered by the player
     * @return The integer value that player enters
     */
    private Integer GetInteger() {

        BufferedReader MyReader = new BufferedReader(new InputStreamReader(System.in));
        String tempString = "";
        Integer myInteger = new Integer(0);
        boolean valid = true;

        do {
            try {
                tempString = MyReader.readLine();
            } catch (IOException IOerror) {
                System.out.println("GetInteger: Read error");
                valid = false;
            }

            try {
                myInteger = Integer.valueOf(tempString);
                valid = true;
            } catch (NumberFormatException NFerror) {
                System.out.println("GetInteger: Number format error");
                valid = false;
            }
        } while (valid == false);

        return myInteger;
    }

    /**
     * This method displays the current list of player cards
     */
    private void DisplayCards() {

        Card tmpCard;

        try {
            // Display all of the cards in Player's hand
            System.out.println(" -Player has:");
            for (int n = 0; n < myHand.size(); n++) {
                tmpCard = (Card) myHand.get(n);
                System.out.println("    " + n + " : " + tmpCard.getName());
            }
            // Display all of the cards in the DEALER's hand
            System.out.println(" -DEALER has:");
            for (int n = 0; n < dealerHand.size(); n++) {
                tmpCard = (Card) dealerHand.get(n);
                System.out.println("    " + n + " : " + tmpCard.getName());
            }
        } catch (Exception e) {
            System.out.println("EXCEPTION-DisplayCards: general error");
        }
    }


    // Fields

    /** The id of the last remote caller of a method. */
    private int lastOID = 0;

    /** The last sequence number of a remote call. */
    private int lastSNR = 0;

    /** The last bet returned by placeBet. */
    private int lastBet = 0;

    /** The last option returned by playerOption. */
    private int lastOption = 0;
}

