// $Id: PlayerServerThread.java,v 1.8 2003/03/14 06:23:19 jerobert Exp $
// PlayerServerThread.java - This is the thread for the server portion of
//               Player Application for the Virtual Casino.
// Authors     - Team5
// Date        - Feb 4, 2003
//

package team5.player;

import java.io.*;
import java.util.*;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.PortableServer.*;
import team5.player.comm.*;
import team5.base.comm.*;

class PlayerServerThread extends Thread {

    private ORB myOrb;
    private POA myRootPOA;
    private PlayerSignals myPlayerSignals;
    private PlayerServer myPlayerServer;
    private Player myPlayerRef;


    // -------------------------------------------------------------------
    /**
     @doc This constructor method for the thread.
     @param none
     @return void
     */
    public PlayerServerThread(String args[], Properties props, PlayerSignals playerSignals) {
        try {
            System.out.println(" ---Setting up the Player Server thread ...");

            this.myPlayerSignals = playerSignals;
            myOrb = ORB.init(args, props);
            myRootPOA = POAHelper.narrow(myOrb.resolve_initial_references("RootPOA"));
            myPlayerServer = new PlayerServer(playerSignals);
            myRootPOA.activate_object(myPlayerServer);
            myPlayerRef = PlayerHelper.narrow(myRootPOA.servant_to_reference(myPlayerServer));
        } catch (Exception e) {
            System.err.println("EXCEPTION-PlayerServerThread(): " + e);
            e.printStackTrace(System.err);
            System.out.println("The Player Server Thread is not initialized.");
        }

    }

    // -------------------------------------------------------------------
    /**
     @doc This run method is starts the thread.
     @param none
     @return void
     */
    public void run() {
        try {
            myPlayerServer.setTableActivity(true);

            System.out.println(" --- Running the Player Server");
            myRootPOA.the_POAManager().activate();
            //myOrb.run();

        } catch (Exception e) {
            System.err.println("EXCEPTION-run: " + e);
            e.printStackTrace(System.err);
            System.out.println("The Player Server Thread is not runing.");
        }
    }

    // -------------------------------------------------------------------
    /**
     @doc This method stops the POA.
     @param none
     @return void
     */
    public void StopService() {
        try {
            myPlayerServer.setTableActivity(false);

            System.out.println(" --- Stopping the Player Server");
            myRootPOA.the_POAManager().deactivate(false, false);

        } catch (Exception e) {
            System.err.println("EXCEPTION-StopService: " + e);
            e.printStackTrace(System.err);
        }
    }

    // -------------------------------------------------------------------
    // -------------------------------------------------------------------
    /**
     @doc This method returns the refrence to the Player service .
     @param none
     @return void
     */
    public Player getPlayerRef() {
        System.out.println(" ---Getting reference to the Player ...");
        return myPlayerRef;
    }

    // -------------------------------------------------------------------
    /**
     @doc This method initializes the number of purchased chips.
     @param int number of chips
     @return void
     */
    public void setChips(int numberOfChips) {
        myPlayerServer.setChips(numberOfChips);
    }

}
