// $Id: FTTablesProxy.java,v 1.4 2003/03/15 18:42:49 lw Exp $
package team5.player.comm;

import team5.base.comm.*;
import org.omg.CORBA.SystemException;

import java.util.Properties;

/**
 * This class acts as in intermediary between the player and the table
 * CORBA stubs. All calls from the TableOperations interface are forwarded
 * to the corresponding Table stub.
 * Each player should have access to exactly one table, therefore this
 * class is implemented as a singleton.
 *
 * @author Team 5
 * @version $Revision: 1.4 $
 */
public class FTTablesProxy implements ITablesProxy {

    /**
     * Get the BankProxy object. Create it if necessary.
     * @return the singleton bank proxy
     */
    public synchronized static FTTablesProxy getInstance(Properties orbprops)
            throws CommException {

        if (theInstance == null) {
            theInstance = new FTTablesProxy(orbprops);
        }
        return theInstance;
    }

    /**
     * Player asks to join any table.
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Enables a player to join a table.</li>
     * <li>Clients: Players</li></ul>
     * @param player The player.
     * @throws CommException
     * @throws InvalidPlayer
     */
    public void joinTable(Player player)
            throws CommException, InvalidPlayer {

        try {
            remoteTables.joinTable(
                    ObjectID.getOID(), SequenceNumber.next(), player);
        } catch (InvalidObjectID e) {
            throw new CommException(e);
        } catch (InvalidSequenceNumber e) {
            throw new CommException(e);
        } catch (SystemException e) {
            try {
                getNewTables();
                remoteTables.joinTable(
                        ObjectID.getOID(), SequenceNumber.next(), player);
            } catch (InvalidObjectID e1) {
                throw new CommException(e1);
            } catch (InvalidSequenceNumber e1) {
                throw new CommException(e1);
            } catch (SystemException e1) {
                throw new CommException(e1);
            }
        }
    }

    /**
     * Player elects to leave a game.
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Enables a player to leave a table.</li>
     * <li>Clients: Players</li></ul>
     * @throws CommException
     */
    public void leaveTable() throws CommException {

        try {
            remoteTables.leaveTable(
                    ObjectID.getOID(), SequenceNumber.next());
        } catch (InvalidObjectID e) {
            throw new CommException(e);
        } catch (InvalidSequenceNumber e) {
            throw new CommException(e);
        } catch (SystemException e) {
            try {
                getNewTables();
                remoteTables.leaveTable(
                        ObjectID.getOID(), SequenceNumber.next());
            } catch (InvalidObjectID e1) {
                throw new CommException(e1);
            } catch (InvalidSequenceNumber e1) {
                throw new CommException(e1);
            } catch (SystemException e1) {
                throw new CommException(e1);
            }
        }
    }


    // Private methods

    /**
     * Create a new TablesProxy.
     */
    private FTTablesProxy(Properties orbprops) throws CommException {

        this.orbprops = orbprops;
        try {
            remoteTables = FTFloorProxy.getInstance(orbprops).
                    getTables(FTFloorProxy.NORMAL);
        } catch (DuplicateValueException e) {
            throw new CommException(e);
        }
    }

    /**
     * Get a new remote tables.
     */
    private void getNewTables() throws CommException {

        try {
            remoteTables = FTFloorProxy.getInstance(orbprops).
                    getTables(FTFloorProxy.FAILOVER);
        } catch (DuplicateValueException e) {
            throw new CommException(e);
        }
    }


    // Fields

    /** The single instance of this class. */
    private static FTTablesProxy theInstance = null;

    /** The Tables stub. */
    Tables remoteTables = null;

    private Properties orbprops;
}

