// $Id: IBankProxy.java,v 1.2 2003/03/06 18:19:10 lw Exp $
package team5.player.comm;

import team5.base.comm.*;

/**
 * This class acts as in intermediary between the player and the bank
 * CORBA stubs. All calls from the BankOperations interface are forwarded
 * to the corresponding Bank stub.
 * Each player should have access to exactly one bank, therefore this
 * class implements the singleton pattern.
 *
 * @author Team 5
 * @version $Revision: 1.2 $
 */
public interface IBankProxy {

    /**
     * The player is purchasing chips from the bank.
     * <ul><li>Category: Baseline.</li>
     * <li>Behavior: The player asks for quantity of chips.
     * <li>Clients:  Player</li></ul>
     * @param quantity The number of chips requested.
     * @throws CommException
     * @throws InvalidQuantity
     */
    void buyChips(int quantity) throws CommException, InvalidQuantity;

    /**
     * The player is selling chips back to the bank.
     * <ul><li>Category: Baseline.</li>
     * <li>Behavior: The player sells all or some of his chips back to the
     *           Bank.</li>
     * <li>Clients:  Player</li></ul>
     * @param quantity The number of chips requested.
     * @throws CommException
     * @throws InvalidQuantity
     */
    void sellChips(int quantity) throws CommException, InvalidQuantity;

    /**
     * The player is selling chips back to the bank.
     * <ul><li>Category: Baseline.</li>
     * <li>Behavior: The player sells all or some of his chips back to the
     *           Bank.</li>
     * <li>Clients:  Player</li></ul>
     * @throws CommException
     */
    void sellAllChips() throws CommException;

    /**
     * The player requests the number of chips he/she owns
     * <ul><li>Category: Baseline.</li>
     * <li>Behavior: The player requests the balance of chips from the bank.</li>
     * <li>Clients:  Player</li></ul>
     * @return current number of chips
     * @throws CommException
     */
    int getChipBalance(int playerID) throws CommException;
}





