// $Id: IFloorProxy.java,v 1.1 2003/02/27 21:04:22 lw Exp $
package team5.player.comm;

import team5.base.comm.Bank;
import team5.base.comm.CommException;
import team5.base.comm.InvalidPlayerName;
import team5.base.comm.Tables;

/**
 * This class acts as in intermediary between the player and the floor
 * CORBA stubs. All calls from the FloorOperations interface are forwarded
 * to the corresponding Floor stub.
 * Each player should have access to exactly one floor, therefore this
 * class is implemented as a singleton.
 *
 * @author Team 5
 * @version $Revision: 1.1 $
 */
public interface IFloorProxy {

    /**
     * Returns an object ID to the caller. This OID is guaranteed to be unique and
     * can be used to identify players or tables.
     * <ul><li>Category: Fault tolerance</li>
     * <li>Behavior: Player clients and tables use this method to obtain a unique ID.
     * <li>Clients: Player, Table</li></ul>
     * @return long The new object ID.
     */
    int getOID() throws CommException;

    /**
     * The new or returning player registers with the system to start
     * <ul><li>Category: Baseline</li>
     * <li>Behavior: A player client uses this method to register for
     *               a session at the casino server.</li>
     * <li>Clients: Player</li></ul>
     * @param playerName The client's Player interface.
     * @return long The player's ID number
     * @throws team5.base.comm.CommException
     */
    int register(String playerName) throws CommException, InvalidPlayerName;

    /**
     * The player asks to leave the game.
     * <ul><li>Category: Baseline</li>
     * <li>Behavior: A player client uses this method to unregister for a session
     *           from the casino server.</li>
     * <li>Clients: Player</li></ul>
     * @throws CommException
     */
    void unregister() throws CommException;
}
