// $Id: ITablesProxy.java,v 1.1 2003/02/27 21:04:22 lw Exp $
package team5.player.comm;

import team5.base.comm.*;
import org.omg.CORBA.SystemException;

/**
 * This class acts as in intermediary between the player and the table
 * CORBA stubs. All calls from the TableOperations interface are forwarded
 * to the corresponding Table stub.
 * Each player should have access to exactly one table, therefore this
 * class is implemented as a singleton.
 *
 * @author Team 5
 * @version $Revision: 1.1 $
 */
public interface ITablesProxy {

    /**
     * Player asks to join any table.
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Enables a player to join a table.</li>
     * <li>Clients: Players</li></ul>
     * @param player The player.
     * @throws CommException
     * @throws InvalidPlayer
     */
    void joinTable(Player player) throws CommException, InvalidPlayer;

    /**
     * Player elects to leave a game.
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Enables a player to leave a table.</li>
     * <li>Clients: Players</li></ul>
     * @throws CommException
     */
    void leaveTable() throws CommException;
}

