// $Id: TablesProxy.java,v 1.8 2003/03/10 02:17:43 lw Exp $
package team5.player.comm;

import team5.base.comm.*;
import org.omg.CORBA.SystemException;

import java.util.Properties;

/**
 * This class acts as in intermediary between the player and the table
 * CORBA stubs. All calls from the TableOperations interface are forwarded
 * to the corresponding Table stub.
 * Each player should have access to exactly one table, therefore this
 * class is implemented as a singleton.
 *
 * @author Team 5
 * @version $Revision: 1.8 $
 */
public class TablesProxy implements ITablesProxy {

    /**
     * Get the BankProxy object. Create it if necessary.
     * @return the singleton bank proxy
     */
    public synchronized static TablesProxy getInstance(Properties orbprops)
        throws CommException , DuplicateValueException {

        if (theInstance == null) {
            theInstance = new TablesProxy(orbprops);
        }
        return theInstance;
    }

    /**
     * Player asks to join any table.
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Enables a player to join a table.</li>
     * <li>Clients: Players</li></ul>
     * @param player The player.
     * @throws CommException
     * @throws InvalidPlayer
     */
    public void joinTable(Player player)
            throws CommException, InvalidPlayer {

        try {
            remoteTables.joinTable(ObjectID.getOID(), SequenceNumber.next(), player);
        } catch (InvalidObjectID e) {
            throw new CommException(e);
        } catch (InvalidSequenceNumber e) {
            throw new CommException(e);
        } catch (SystemException e) {
            throw new CommException(e);
        }
    }

    /**
     * Player elects to leave a game.
     * <ul><li>Category: Baseline </li>
     * <li>Behavior: Enables a player to leave a table.</li>
     * <li>Clients: Players</li></ul>
     * @throws CommException
     */
    public void leaveTable() throws CommException {

        try {
            remoteTables.leaveTable(ObjectID.getOID(), SequenceNumber.next());
        } catch (InvalidObjectID e) {
            throw new CommException(e);
        } catch (InvalidSequenceNumber e) {
            throw new CommException(e);
        } catch (SystemException e) {
            throw new CommException(e);
        }
    }


    // Private methods

    /**
     * Create a new TablesProxy.
     */
    private TablesProxy(Properties orbprops) throws CommException, DuplicateValueException {

        remoteTables = FloorProxy.getInstance(orbprops).getTables();
    }


    // Fields

    /** The single instance of this class. */
    private static TablesProxy theInstance = null;

    /** The Tables stub. */
    private Tables remoteTables = null;
}

