// $Id: Replica.java,v 1.9 2003/03/15 15:01:15 lw Exp $
package team5.repman;

import org.omg.CORBA.SystemException;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.InvalidName;
import org.omg.CosNaming.NamingContextPackage.NotFound;
import team5.base.comm.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * Replica
 * @author Team 5
 * @version $Revision: 1.9 $
 * */
public class Replica extends Thread {

    // Nested classes

    class Exec extends Thread {

        class StreamGobbler extends Thread {

            StreamGobbler(InputStream is, String type) {

                this.is = is;
                this.type = type;
            }

            public void run() {

                try {
                    InputStreamReader isr = new InputStreamReader(is);
                    BufferedReader br = new BufferedReader(isr);
                    String line = null;
                    while ((line = br.readLine()) != null) {
                        Verbose.println(type + ">" + line);
                    }
                } catch (IOException ioe) {
                    System.err.println("Error: Internal Error: I/O exception in StreamGobbler: ");
                }
            }

            private InputStream is;
            private String type;
        } // class StreamGobbler

        public void run() {

            try {
                String osName = System.getProperty("os.name");
                String[] cmd = new String[5];

                if (osName.equals("Windows 2000") || osName.equals("Windows XP")) {
                    cmd[0] = "cmd.exe ";
                    cmd[1] = "/C ";
                    cmd[2] = "C:\\cygwin\\bin\\ssh";
                    cmd[3] = "lw@" + hostname + ".ece.cmu.edu";
                    cmd[4] = startCommand;
                } else if (osName.equals("Linux")) {
                    cmd[0] = "ssh";
                    cmd[1] = hostname;
                    cmd[2] = startCommand;
                    cmd[3] = "";
                    cmd[4] = "";
                } else {
                    System.err.println("Fatal: Unknown Operating System " + osName);
                    return;
                }

                Runtime rt = Runtime.getRuntime();
                Verbose.println("---Execing " + cmd[0] + " " + cmd[1] + " " + cmd[2] +
                        cmd[3] + cmd[4]);
                Process proc = rt.exec(cmd);
                // any error message?
                StreamGobbler errorGobbler = new
                        StreamGobbler(proc.getErrorStream(), "Floor" + number + ".err");

                // any output?
                StreamGobbler outputGobbler = new
                        StreamGobbler(proc.getInputStream(), "Floor" + number + ".out");

                // kick them off
                errorGobbler.start();
                outputGobbler.start();

                // any error???
                int exitVal = proc.waitFor();
                Verbose.println("---ExitValue(" + number + "): " + exitVal);
            } catch (Throwable t) {
                System.err.println("Error: Internal error: Throwable in Exec.run: " + t);
                //t.printStackTrace();
            }
        }
    } // class Exec


    // Public methods

    /**
     * The constructor for the Replica class
     */
    public Replica(Repman repman, NamingContextExt namingContext,
                   String hostname, int number, long sleeptime, String cmd)
            throws InterruptedException, CommException {

        this.repman = repman;
        this.namingContext = namingContext;
        this.hostname = hostname;
        this.number = number;
        this.sleeptime = sleeptime;
        startCommand = cmd + " -n " + number;
        isPrimary = false;
        exec();
    }

    /**
     * The thread pings the floor continuously.
     */
    public void run() {

        while (true) {
            try {
                sleep(sleeptime);
                remoteFloor.ping();
            } catch (InterruptedException ie) {
                Verbose.println("---sleep interrupted for replica #" + number);
            } catch (SystemException e) {
                System.out.println("Info: Failed to ping replica #" + number);
                if (isPrimary) {
                    isPrimary = false;
                    repman.replacePrimary(this);
                }
                repman.notifyReplicaDeath(this);
                break;
            }
        }
    }

    /**
     * Gets the number of this replica
     * @return the number of this replica
     */
    public int getNumber() {

        return number;
    }

    /**
     * Gets the host name this replica runs on
     * @return the host name
     */
    public String getHostName() {

        return hostname;
    }

    /**
     * A wrapper for the Casion server's activate method
     */
    public void activate() throws CommException {

        try {
            this.isPrimary = true;
            remoteFloor.activate();
        } catch (SystemException e) {
            Verbose.println("---EXC: " + e);
            //e.printStackTrace();
            System.err.println("Error: Could not activate new primary");
            throw new CommException("Could not activate primary #" + number, e);
        }
    }

    /**
     * Call the operating system to start the casino server
     */
    private void exec() throws InterruptedException, CommException {

        try {
            // wait 30 sec for server to start up
            wd.enable(30);
            new Exec().start();

            while (true) {
                Thread.sleep(1000);
                // see once a second if it's running
                try {
                    remoteFloor = FloorHelper.narrow(
                            namingContext.resolve_str("Floor" + number));
                    wd.cancel();
                    break;
                } catch (SystemException e) {
                    wd.cancel();
                    throw new CommException();
                } catch (NotFound e) {
                    // ignore!
                } catch (CannotProceed e) {
                    wd.cancel();
                    throw new CommException();
                } catch (InvalidName e) {
                    wd.cancel();
                    throw new CommException();
                }
            }
        } catch (InterruptedException e) {
            wd.cancel();
            System.err.println("Error: Timer expired during startup of 'Floor" + number + "'");
            throw e;
        }
    }


    // Fields

    /** The replication manager that started this. */
    private Repman repman;

    /**  The instance number of this replica. */
    private int number;

    /** The namingContext to use. */
    private NamingContextExt namingContext;

    /** The name of the host where the replica should run. */
    private String hostname;

    /** The time to wait between pings. */
    long sleeptime;

    /** Reference to the running replica. */
    private Floor remoteFloor;

    /** The command line string to start a server. */
    private String startCommand = "";

    /** Indicates if this replica is the primary. */
    private boolean isPrimary = false;

    /** Thw watchdog generates timeout interrupts. */
    private Watchdog wd = new Watchdog(this);
}
